locals {
  compaction_job_config        = jsondecode(var.extra_environment.COMPACTION_JOB_CONFIG)
  cleaning_job_config          = jsondecode(var.extra_environment.CLEANING_JOB_CONFIG)
  archiving_job_config         = jsondecode(var.extra_environment.ARCHIVING_JOB_CONFIG)
  delete_partitions_job_config = jsondecode(var.extra_environment.DELETE_PARTITIONS_JOB_CONFIG)
  clustering_plan_job_config   = jsondecode(var.extra_environment.CLUSTERING_PLAN_JOB_CONFIG)
  clustering_job_config        = jsondecode(var.extra_environment.CLUSTERING_JOB_CONFIG)
  common_hudi_config           = jsondecode(var.extra_environment.COMMON_HUDI_CONFIG)
  sade_uds_alerts_lib_version  = lookup(local.compaction_job_config.glue_config, "lib_version", var.default_sade_uds_alerts_lib_version)

  # This is a workaround for not having this variable in the team cluster.
  # Metrics will fail in the team cluster, but the job will succeed.
  wavefront_url = "https://sbu.wavefront.com"

  glue_version = "4.0"

  # Jars are commonized across all jobs, even if there are some that do not need all of them
  extra_jars = join(",",
    concat(
      # Jars provided in values.yaml file
      # These are 3rd party jars
      # Jars are assumed to be order to define loading into Spark
      formatlist("s3://${var.processing_bucket_name}/jars/%s", split(",", var.hudi_jars)),

      # Hudi extensions library
      ["s3://${var.processing_bucket_name}/lib/hudi-extensions-${var.hudi_extensions_lib_version}.jar"],
      # Spark Ingress library
      ["s3://${var.processing_bucket_name}/lib/spark-ingress-${var.spark_ingress_lib_version}.jar"]
    )
  )

  compaction_hudi_config        = merge(local.common_hudi_config, local.compaction_job_config.hudi_config)
  cleaning_hudi_config          = merge(local.common_hudi_config, local.cleaning_job_config.hudi_config)
  archiving_hudi_config         = merge(local.common_hudi_config, local.archiving_job_config.hudi_config)
  delete_partitions_hudi_config = merge(local.common_hudi_config, local.delete_partitions_job_config.hudi_config)
  clustering_plan_hudi_config   = merge(local.common_hudi_config, local.clustering_plan_job_config.hudi_config)
  clustering_hudi_config        = merge(local.common_hudi_config, local.clustering_job_config.hudi_config)
}

data "aws_iam_role" "job_role" {
  name = var.iam_role_name
}

resource "aws_glue_job" "sade_uds_alerts_compaction" {
  name                   = "${var.name_prefix}-sade-uds-alerts-compaction"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.compaction_job_config.glue_config.timeout
  number_of_workers      = local.compaction_job_config.glue_config.num_workers
  worker_type            = local.compaction_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/compaction/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.compaction_hudi_config)
    "--job_type"                         = "COMPACT"
    "--extra-jars"                       = local.extra_jars
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }

  execution_property {
    max_concurrent_runs = local.compaction_job_config.glue_config["max_concurrency"]
  }
}

resource "aws_glue_job" "sade_uds_alerts_cleaning" {
  name                   = "${var.name_prefix}-sade-uds-alerts-cleaning"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.cleaning_job_config.glue_config.timeout
  number_of_workers      = local.cleaning_job_config.glue_config.num_workers
  worker_type            = local.cleaning_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/cleaning/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.cleaning_hudi_config)
    "--job_type"                         = "CLEAN"
    "--extra-jars"                       = local.extra_jars
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }

  execution_property {
    max_concurrent_runs = local.cleaning_job_config.glue_config["max_concurrency"]
  }
}

resource "aws_glue_job" "sade_uds_alerts_archiving" {
  name                   = "${var.name_prefix}-sade-uds-alerts-archiving"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.archiving_job_config.glue_config.timeout
  number_of_workers      = local.archiving_job_config.glue_config.num_workers
  worker_type            = local.archiving_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/archiving/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.archiving_hudi_config)
    "--job_type"                         = "ARCHIVE"
    "--extra-jars"                       = local.extra_jars
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }

  execution_property {
    max_concurrent_runs = local.archiving_job_config.glue_config["max_concurrency"]
  }
}

resource "aws_glue_job" "sade_uds_alerts_delete_partitions" {
  name                   = "${var.name_prefix}-sade-uds-alerts-delete-partitions"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.delete_partitions_job_config.glue_config.timeout
  number_of_workers      = local.delete_partitions_job_config.glue_config.num_workers
  worker_type            = local.delete_partitions_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/delete_partitions/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.delete_partitions_hudi_config)
    "--hudi_job_args"                    = jsonencode(local.delete_partitions_job_config.hudi_job_args)
    "--job_type"                         = "DELETE_PARTITION"
    "--extra-jars"                       = local.extra_jars
    "--src_account_id"                   = var.extra_environment.SRC_ACCOUNT_ID
    "--delete_partitions_hours_lookback" = local.delete_partitions_job_config.delete_partitions_hours_lookback
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }

  execution_property {
    max_concurrent_runs = local.delete_partitions_job_config.glue_config["max_concurrency"]
  }
}

resource "aws_glue_job" "sade_uds_alerts_clustering_plan" {
  name                   = "${var.name_prefix}-sade-uds-alerts-clustering-plan"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.clustering_plan_job_config.glue_config.timeout
  number_of_workers      = local.clustering_plan_job_config.glue_config.num_workers
  worker_type            = local.clustering_plan_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/clustering_plan/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.clustering_plan_hudi_config)
    "--job_type"                         = "CLUSTERING_PLAN"
    "--extra-jars"                       = local.extra_jars
    "--downstream_job"                   = aws_glue_job.sade_uds_alerts_clustering.name
    "--clustering_plan_hours_lookback"   = local.clustering_plan_job_config.clustering_plan_hours_lookback
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }
}

resource "aws_glue_job" "sade_uds_alerts_clustering" {
  name                   = "${var.name_prefix}-sade-uds-alerts-clustering"
  tags                   = var.tags
  glue_version           = local.glue_version
  role_arn               = data.aws_iam_role.job_role.arn
  max_retries            = 0
  timeout                = local.clustering_job_config.glue_config.timeout
  number_of_workers      = local.clustering_job_config.glue_config.num_workers
  worker_type            = local.clustering_job_config.glue_config.worker_type
  security_configuration = aws_glue_security_configuration.sade_uds_alerts.name

  default_arguments = {
    "--table_name"                       = var.extra_environment.TABLE_NAME
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.processing_bucket_name}/lib/sade_uds_alerts-${local.sade_uds_alerts_lib_version}-py3-none-any.whl"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/sade_uds_alerts/clustering/"
    "--env"                              = var.extra_environment.ENV_NAME
    "--wavefront_url"                    = local.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    "--hudi_config"                      = jsonencode(local.clustering_hudi_config)
    "--job_type"                         = "CLUSTERING"
    "--extra-jars"                       = local.extra_jars
    "--extra-files"                      = "s3://${var.processing_bucket_name}/scheduler/fair-scheduler.xml"
  }

  command {
    script_location = "s3://${var.processing_bucket_name}/jobs/sade_uds_alerts/table_services/glue_handler_sade_uds_alerts-${local.sade_uds_alerts_lib_version}.py"
  }
}
